import base64
import logging
from typing import Dict, Optional, Any

logger = logging.getLogger("konglambdamessagehandler")


class LambdaResponse(object):
    def __init__(
        self,
        status: int,
        response: Optional[Any],
        headers: Optional[Dict[str, str]],
    ):
        headers = headers or {}
        self.response: Dict[str, Any] = {"statusCode": status}
        if response:
            content_type = headers.get("Content-Type")
            content_encoding = headers.get("Content-Encoding")
            if (
                (content_type is not None and not content_type.startswith("application/json"))
                or (content_encoding is not None and content_encoding.startswith("gzip"))
            ):
                self.response["body"] = base64.b64encode(response)
                logger.info("Base64 Body size %s", len(self.response["body"]))
                headers["x-lambda-base64-body"] = "true"
            else:
                self.response["body"] = response
        if headers:
            self.response["headers"] = {}
            for header in headers:
                self.response["headers"][header] = headers[header]



import pytest


def parametrize(name, params):
    names = name.split(",") if isinstance(name, str) else name

    ids = [f"{key}" for key in params]
    values = [tuple(params[key].values()) for key in params]

    return pytest.mark.parametrize(names, values, ids=ids)



import gzip
import logging
from typing import Dict, Tuple, Optional

from httpclient import HttpRequest
from httpmessagehandler import HttpMessageHandler

from ._KongRequest import KongRequest
from ._LambdaResponse import LambdaResponse

logger = logging.getLogger("konglambdamessagehandler")

_DEFAULT_COMPRESSION_THRESHOLD_BYTES = 1024 * 1024  # 1MB


class KongLambdaMessageHandler:

    def __init__(
        self,
        http_message_handler: HttpMessageHandler,
        compression_threshold: Optional[int] = None,
    ) -> None:
        self.http_message_handler = http_message_handler
        self.compression_threshold = compression_threshold or _DEFAULT_COMPRESSION_THRESHOLD_BYTES

    def handle_messages(self, event: Dict) -> Dict:
        http_request = self._parse_http_request(event)
        http_response = self.http_message_handler.handle_request(request=http_request)
        body, headers = self._compress_body(
            body=http_response.body,
            headers=http_response.headers
        )
        response = LambdaResponse(
            status=http_response.status_code,
            response=body,
            headers=headers,
        )
        return response.response

    @staticmethod
    def _parse_http_request(event):
        request = KongRequest(event)
        http_request = HttpRequest(
            url=request.path,
            method=request.method,
            query=request.args,
            headers=request.headers,
            body=request.data,
        )
        return http_request

    def _compress_body(self, body: bytes, headers: Dict[str, str]) -> Tuple[bytes, dict]:
        if "Content-Encoding" not in headers and len(body) > self.compression_threshold:
            logger.info("Body size %s > %s, compressing.", len(body), self.compression_threshold)
            body = gzip.compress(data=body)
            logger.info("Compressed body size %s", len(body))
            headers["Content-Encoding"] = "gzip"
        return body, headers



import json
from typing import Dict


class KongRequest(object):
    def __init__(self, event: Dict):
        self.path = str(event['request_uri']).split('?')[0]
        self.method = event['request_method']
        self.args = event['request_uri_args']
        self.headers = event['request_headers']
        self.data = self._parse_request_body(event)

    @staticmethod
    def _parse_request_body(event: Dict) -> bytes:
        request_body = event.get('request_body')
        if request_body is not None:
            if isinstance(request_body, dict):
                request_body = json.dumps(request_body, default=str)
            if not isinstance(request_body, str):
                request_body = str(request_body)
            return request_body.encode("utf-8")
        return b""
